@@ -26,17 +26,19 @@ import shutil
 from time import sleep
 from calculate.core.server.func import MethodsInterface
 from calculate.core.server.admin import Admins
+from calculate.lib.utils.mount import isMount
 from calculate.lib.utils.files import (pathJoin,
-                                       isMount, process, listDirectory,
+                                       process, listDirectory, writeFile,
                                        checkUtils, readFile, find, copyWithPath,
                                        readLinesFile, getProgPath)
+from collections import deque
+import calculate.lib.utils.device as device
 from calculate.lib.utils.device import (detectDeviceForPartition,
-                                        getUdevDeviceInfo, refreshLVM,
-                                        refreshUdev, countPartitions)
+                                        countPartitions)
+from calculate.lib.utils.partition import DiskFactory
 from datavars import DataVarsInstall
-from calculate.install.variables.autopartition import AutoPartition
 
-from distr import DistributiveError
+from distr import DistributiveError, PartitionDistributive
 from subprocess import Popen, PIPE, STDOUT
 from itertools import *
 
@@ -59,9 +61,9 @@ class Install(MethodsInterface):
     def __init__(self):
         self.clVars = None
         # refresh information about LVM
-        refreshLVM()
+        device.lvm.refresh()
         # refresh information about device in udevadm info
-        refreshUdev()
+        device.udev.refresh()
 
     def initVars(self, datavars=None):
         """Primary initialization of variables"""
@@ -113,10 +115,10 @@ class Install(MethodsInterface):
         partition_table = self.clVars.Select('os_device_table',
                                              where='os_device_dev',
                                              eq=disk, limit=1)
+        info = device.udev.get_device_info(name=partition)
         partition_number = (
-            getUdevDeviceInfo(name=partition).get('ID_PART_ENTRY_NUMBER', '') or
-            getUdevDeviceInfo(name=partition).get('UDISKS_PARTITION_NUMBER', '')
-        )
+            info.get('ID_PART_ENTRY_NUMBER', '') or
+            info.get('UDISKS_PARTITION_NUMBER', ''))
 
         device_partition_count = countPartitions(device_name)
         if device_name and not partition_number:
@@ -234,6 +236,9 @@ class Install(MethodsInterface):
             self.install_grub_uefi(cmd_grub_install, prefix_boot, target)
         # не UEFI установка
         else:
+            if self.clVars.Get('os_uefi'):
+                self.update_efi_fstab()
+                self.mount_efi_fstab()
             self.install_grub_biosboot(cmd_grub_install, prefix_boot, target)
 
     def install_grub_biosboot(self, cmd_grub_install, prefix_boot, target):
@@ -270,42 +275,132 @@ class Install(MethodsInterface):
                 raise DistributiveError(
                     _("Failed to install the bootloader"))
 
+    def update_efi_fstab(self):
+        fstab_fn = pathJoin(self.clVars.Get('cl_chroot_path'), "/etc/fstab")
+        re_efi_record = re.compile("^(# /boot/efi|\S+\s/boot/efi).*\n",
+                                   flags=re.M)
+
+        efidata = self.clVars.Get('os_install_fstab_efi_conf')
+
+        if path.exists(fstab_fn):
+            data = readFile(fstab_fn)
+            m = re_efi_record.search(data)
+            newdata = re_efi_record.sub("", data)
+            if efidata:
+                if m:
+                    newdata = "%s%s\n%s" % (newdata[:m.start()],
+                                            efidata, newdata[m.start():])
+                else:
+                    newdata = "%s%s\n" % (newdata, efidata)
+            if data != newdata:
+                with writeFile(fstab_fn) as f:
+                    f.write(newdata)
+
+    def format_efi(self):
+        formatdisk = [(dev, fs)
+            for dev, mp, fs, make in self.clVars.ZipVars(
+                'os_install_disk_dev',
+                'os_install_disk_mount',
+                'os_install_disk_format',
+                'os_install_disk_perform_format')
+            if mp.startswith('/boot/efi') and make == 'on'
+        ]
+        if formatdisk:
+            self.startTask(_("Formatting the partitions"), progress=True,
+                           num=len(formatdisk))
+            i = 1
+            for dev, fs in formatdisk:
+                self.formatPartition(dev, format="vfat")
+                self.setProgress(i)
+                i += 1
+            self.endTask(True)
+
+    def umountDirectory(self, directory):
+        return PartitionDistributive(None)._umountDirectory(directory)
+
+    def makeDirectory(self, directory):
+        return PartitionDistributive(None)._makeDirectory(directory)
+
+    def mountToDirectory(self, dev, mp):
+        return PartitionDistributive(None)._mountToDirectory(dev, mp)
+
+    def formatPartition(self, dev, format="ext4", label=""):
+        return PartitionDistributive(None).formatPartition(dev, format, label)
+
+    def mount_efi_fstab(self):
+        oldmp_efi = self.clVars.select('os_disk_mount',
+                                       os_disk_mount__startswith="/boot/efi")
+        for dev, mp in self.clVars.ZipVars('os_install_disk_dev',
+                                           'os_install_disk_mount'):
+            if mp.startswith("/boot/efi"):
+                curdev = isMount(mp)
+                if curdev != dev:
+                    if curdev:
+                        self.umountDirectory(mp)
+                    self.makeDirectory(mp)
+                    self.mountToDirectory(dev, mp)
+                if mp in oldmp_efi:
+                    oldmp_efi.remove(mp)
+        for mp in oldmp_efi:
+            self.umountDirectory(mp)
+
     def install_grub_uefi(self, cmd_grub_install, prefix_boot, target):
+        if self.clVars.Get('cl_action') != "system":
+            self.format_efi()
+            self.update_efi_fstab()
+            self.mount_efi_fstab()
+        efidirs = [x for x in self.clVars.Get('os_install_disk_mount')
+                   if x.startswith('/boot/efi')]
+        if len(efidirs) > 1:
+            labels = ["calculate%d" % i for i in range(1, len(efidirs) + 1)]
+        else:
+            labels = ["calculate"]
+        for efiname, efidir in reversed(zip(labels, efidirs)):
+            self._install_grub_uefi(cmd_grub_install, prefix_boot, target,
+                                    efidir, efiname)
+        # удаляем устаревшие
+        efi_boot_mgr = getProgPath('/usr/sbin/efibootmgr')
+        p_efibootmgr = process(efi_boot_mgr, "-v")
+        data = p_efibootmgr.read()
+        for num, label in re.findall(r"Boot(\d+).*(calculate\d*)", data):
+            if label not in labels:
+                process(efi_boot_mgr, "-b", num, "-B").success()
+
+    def _install_grub_uefi(
+            self, cmd_grub_install, prefix_boot, target, efidir, efiname):
         """
         Установить grub с UEFI загрузчиком
         """
+        efifulldir = pathJoin(target.getDirectory(), efidir)
         grub_params = [
             "--boot-directory=%s" % pathJoin(
                 prefix_boot,
                 target.getBootDirectory()),
+            "--bootloader-id=%s" % efiname,
             "--target=x86_64-efi",
-            "--efi-directory=%s" %
-            target.getEfiDirectory(),
+            "--efi-directory=%s" % efifulldir,
             "--force"]
         # проверяем наличие в nv-ram нужной нам записи для исключения повтора
         efi_boot_mgr = getProgPath('/usr/sbin/efibootmgr')
         efi_disk = self.clVars.Select("os_install_disk_dev",
                                       where="os_install_disk_mount",
-                                      eq="/boot/efi", limit=1)
+                                      eq=efidir, limit=1)
         if efi_disk:
-            efi_uuid = getUdevDeviceInfo(
+            efi_uuid = device.udev.get_device_info(
                 name=efi_disk).get("ID_PART_ENTRY_UUID", "")
             if efi_uuid:
                 p_efibootmgr = process(efi_boot_mgr, "-v")
                 data = p_efibootmgr.read()
-                if re.search(r"Boot.*calculate.*GPT,{uuid}.*{efipath}".format(
+                if re.search(r"Boot.*{label}\s.*GPT,{uuid}.*{efipath}".format(
+                        label=efiname,
                         uuid=efi_uuid,
-                        efipath=r"\\EFI\\calculate\\grubx64.efi"),
+                        efipath=r"\\EFI\\%s\\grubx64.efi" % efiname),
                         data, flags=re.M | re.I):
                     grub_params.append("--no-nvram")
         # в случае установки на usb-hdd EFI загрузчик не прописывается
         # в efivars
         if self.clVars.Get('os_install_root_type') == 'usb-hdd':
             grub_params.append("--removable")
-        if self.clVars.Get('cl_action') != "system" and \
-                not isMount('/boot/efi'):
-            raise DistributiveError(_("Failed to install the bootloader. "
-                                      "/boot/efi is not mounted."))
         grub_process = process(cmd_grub_install,
                                *grub_params, stderr=STDOUT,
                                envdict=os.environ)
@@ -317,7 +412,7 @@ class Install(MethodsInterface):
         # запись создать не удалось
         dmesg = getProgPath('/bin/dmesg')
         if efi_boot_mgr and dmesg:
-            if not re.search('Boot.*calculate',
+            if not re.search('Boot.*%s\s' % efiname,
                              process(efi_boot_mgr).read(), re.M) and \
                     re.search('efivars.*set_variable.*failed',
                               process(dmesg).read(), re.M):
@@ -365,7 +460,7 @@ class Install(MethodsInterface):
         path_gl_modules = path.join(self.clVars.Get('cl_chroot_path'),
                                     'usr/lib/opengl')
         open_gl_env = path.join(self.clVars.Get('cl_chroot_path'),
-                                'etc/env.d/03opengl')
+                                'etc/env.d/000opengl')
 
         open_gl_mods = filter(lambda x: x != "global",
                               listDirectory(path_gl_modules))
@@ -420,28 +515,89 @@ class Install(MethodsInterface):
                                       where='os_disk_mount',
                                       eq='/', limit=1)
         try:
-            scheduler_path = (
-                "/sys%s/queue/scheduler" %
-                (getUdevDeviceInfo(name=root_dev).get('DEVPATH', '')))
-            if path.exists(scheduler_path):
-                open(scheduler_path, 'w').write(scheduler)
+            sysname = device.udev.get_syspath(name=root_dev)
+            if device.sysfs.exists(sysname, device.sysfs.Path.BlockScheduler):
+                device.sysfs.write(
+                    sysname, device.sysfs.Path.BlockScheduler, scheduler)
         except Exception:
             raise InstallError(_("Unable to change the I/O scheduler"))
         return True
 
-    def autopartition(self, table, devices, data, lvm, lvm_vgname, bios_grub,
-                      bios_grub_size):
+    def clearLvm(self, devices):
+        dv = self.clVars
+
+        def get_vgs():
+            for pv, vg, pvbase in dv.ZipVars('os_lvm_pvname',
+                                             'os_lvm_vgname',
+                                             'os_lvm_pvname_parent'):
+                if (pv in devices or
+                        any(x in devices for x in pvbase.split(','))):
+                    yield vg
+
+        remove_vgs = set(get_vgs())
+        remove_pvs = set(self.clVars.select('os_lvm_pvname',
+                                            os_lvm_vgname__in=remove_vgs))
+        remove_lvs = set(self.clVars.select('os_lvm_vgname', 'os_lvm_lvname',
+                                            os_lvm_vgname__in=remove_vgs))
+
+        failed = False
+        for vg, lv in sorted(remove_lvs):
+            failed |= not device.lvm.remove_lv(vg, lv)
+        for vg in sorted(remove_vgs):
+            failed |= not device.lvm.remove_vg(vg)
+        for pv in sorted(remove_pvs):
+            failed |= not device.lvm.remove_pv(pv)
+        return not failed
+
+    def clearRaid(self, devices):
+        dv = self.clVars
+
+        def generate_remove_raids():
+            for raid, parents in dv.select('os_device_dev', 'os_device_parent',
+                                           os_device_type__startswith="raid"):
+                parents = parents.split(',')
+                if any(x in devices for x in parents):
+                    yield raid, set(parents)
+
+        remove_raids = deque(generate_remove_raids())
+
+        failed = False
+        while remove_raids:
+            raid, parents = remove_raids.popleft()
+            # если среди прочих удаляемых RAID есть те, которые используют
+            # текущий - откладываем его в конец
+            if any(raid in _parents for _raid, _parents in remove_raids):
+                remove_raids.append((raid, parents))
+            else:
+                failed |= not device.raid.remove_raid(raid)
+
+        return not failed
+
+    def wait_devices(self, disks):
         """
-        Авторазметка диска с таблицей разделов 'table', диски указываются
-        'device', параметры таблицы 'data', 'lvm' использование LVM,
-        'lvm_vgname' название группы томов LVM, bios_grub - создавать
-        bios_grub раздел, bios_grub_size - раздел bios grub раздела в байтах
+        Ожидание одного из указанных устройств
         """
-        ap = AutoPartition()
-        ap.clearLvm(devices, self.clVars)
-        ap.clearRaid(devices, self.clVars)
-        ap.recreateSpace(table, devices, data, lvm,
-                         lvm_vgname, bios_grub, bios_grub_size)
+        for waittime in (0.1, 0.2, 0.5, 1, 2, 4):
+            disks = [x for x in disks if x and not path.exists(x)]
+            if not disks:
+                break
+            else:
+                sleep(waittime)
+
+        if disks:
+            raise InstallError(
+                _("Failed to found partition %s after creating "
+                  "the partition table")
+                % ",".join(disks))
+
+    def autopartition(self, scheme_builder, devices, disks):
+        """
+        Авторазметка диска входящая переменная - SchemeBuilder
+        """
+        self.clearLvm(devices)
+        self.clearRaid(devices)
+        scheme_builder.process(DiskFactory())
+        self.wait_devices(disks)
         return True
 
     def format(self, target):
