@@ -67,7 +67,8 @@ class Wsdl(WsdlBase):
             lambda group: group(_("Language and locale"),
                                 image="welcome",
                                 normal=('os_install_locale_lang',
-                                        'os_install_clock_timezone')),
+                                        'os_install_clock_timezone'),
+                                expert=('os_install_locale_keyboard_layout',)),
             lambda group: group(_("Distribution"),
                                 normal=('cl_image_filename',),
                                 hide=('cl_image_linux_shortname',
@@ -102,12 +103,16 @@ class Wsdl(WsdlBase):
                                         'os_install_uefi')),
             lambda group: group(_("Network settings"),
                                 normal=(
-                                    'os_install_net_conf',
+                                    'cl_network_migrate_set',),
+                                expert=('os_install_net_conf',
                                     'os_install_net_data',
-                                    'os_install_net_fqdn', 'os_install_ntp'),
-                                expert=('os_install_net_dns',
-                                        'os_install_net_dns_search',
-                                        'os_install_net_route_data')),
+                                    'os_install_net_fqdn', 'os_install_ntp',
+                                    'os_install_net_dns',
+                                    'os_install_net_dns_search',
+                                    'os_install_net_route_data'),
+                                expert_label=_(
+                                    "Click to select network settings")
+                                ),
             lambda group: group(_("Users"),
                                 normal=(
                                     'cl_migrate_root_pwd_plain',
@@ -237,6 +242,7 @@ class Wsdl(WsdlBase):
             'native_error': (
                 VariableError, DataVarsError, install.InstallError),
             'setvars': {'cl_action!': 'merge', 'cl_merge_pkg!': [None],
+                        'cl_network_migrate_set': 'off',
                         'cl_merge_set!': "on", 'cl_setup': 'network'},
             'groups': [
                 lambda group: group(_("Network"),
@@ -269,7 +275,8 @@ class Wsdl(WsdlBase):
             'setvars': {'cl_action!': 'merge', 'cl_live': 'off'},
             'groups': [
                 lambda group: group(_("Update system settings"),
-                                    normal=('cl_live',),
+                                    normal=('cl_live',
+                                            'cl_network_configure_set'),
                                     expert=(
                                         'cl_templates_locate',
                                         'cl_dispatch_conf',
@@ -349,6 +356,7 @@ class Wsdl(WsdlBase):
                                     normal=('os_install_locale_lang',
                                             'os_install_clock_timezone'),
                                     expert=(
+                                        'os_install_locale_keyboard_layout',
                                         'cl_templates_locate',
                                         'cl_dispatch_conf',
                                         'cl_verbose_set'),
